/*
 * Copyright (C) 2004 emuWorks
 * http://games.technoplaza.net/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package chtdecoder.datastructures;

import chtdecoder.exceptions.InvalidAddressException;
import chtdecoder.exceptions.InvalidValueException;
import chtdecoder.exceptions.NameTooLongException;

/**
 * An interface to represent a ZSNES CHT code.
 *
 * @author John David Ratliff
 * @version 1.1, 09/14/04
 */
public interface CHTCode {
    /**
     * Asks if this CHTCode is active (turned ON).
     *
     * @return true if this CHTCode is active; false otherwise.
     */
    public boolean isActive();

    /**
     * Sets whether this CHTCode is active or not.
     *
     * @param active true for active (ON); false for inactive (OFF).
     */
    public void setActive(boolean active);

    /**
     * Gets the address of this CHTCode.
     *
     * @return The address.
     */
    public int getAddress();

    /**
     * Sets the address of this CHTCode.
     *
     * @param address The address.
     *
     * @throws InvalidAddressException if the address is outside the range
     *         0-16777215 (2^24 - 1).
     */
    public void setAddress(int address) throws InvalidAddressException;

    /**
     * Gets the value of this CHTCode.
     *
     * @return The value.
     */
    public int getValue();

    /**
     * Sets the value of this CHTCode.
     *
     * @param value The value.
     *
     * @throws InvalidValueException if the value is outside the range 0-255
     *         (2^8 - 1).
     */
    public void setValue(int value) throws InvalidValueException;

    /**
     * Gets the name of this CHTCode.
     *
     * @return The name.
     */
    public String getName();

    /**
     * Sets the name of this CHTCode (18 characters max).
     *
     * @param name The name.
     *
     * @throws NameTooLongException if the new name is longer than 18
     *         characters.
     */
    public void setName(String name) throws NameTooLongException;
}
