/*
 * Copyright (C) 2004 emuWorks
 * http://games.technoplaza.net/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package chtdecoder.tools;

import java.io.File;

import chtdecoder.datastructures.CHTCodeIterator;

/**
 * An abstract implementation of the CHTFileReader class.
 *
 * @author John David Ratliff
 * @version 1.2, 10/21/04
 */
public abstract class AbstractCHTFileReader implements CHTFileReader {
    /**
     * A factory method for creating CHTCodeIterators.
     *
     * @param chtFile The .CHT File to read.
     * @param force Whether to force opening of potentially invalid .CHT
     *        files.
     *
     * @return A new CHTFileReader
     */
    public static CHTCodeIterator readCHTFile(File chtFile, boolean force) {
        BasicCHTFileReader instance = BasicCHTFileReader.getInstance();
        instance.readFile(chtFile, force);

        return instance.iterator();
    }

    /**
     * An imperfect set of checks for verifying a file is a .CHT file.
     *
     * @param chtFile The File to test.
     *
     * @return returns true if the File is likely a .CHT file; false
     *         otherwise.
     */
    public static boolean isValidCHTFile(File chtFile) {
        // .CHT files should end in .CHT
        if (!chtFile.getName().toLowerCase().endsWith(".cht")) {
            return false;
        }

        // .CHT files should exist
        if (!chtFile.exists()) {
            return false;
        }

        // file length should be a multiple of 28 bytes
        if ((chtFile.length() % 28) != 0) {
            return false;
        }

        return true;
    }
}
