/*
 * Copyright (C) 2004 emuWorks
 * http://games.technoplaza.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package chtdecoder.view;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import java.net.URI;
import java.net.URISyntaxException;

import java.util.Enumeration;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.TransferHandler;

import javax.swing.filechooser.FileFilter;

import chtdecoder.CHTDecoder;

import chtdecoder.datastructures.CHTCode;
import chtdecoder.datastructures.CHTCodeIterator;

import chtdecoder.exceptions.CHTFileException;

import chtdecoder.tools.AbstractCHTFileReader;

/**
 * An applet GUI for the CHTDecoder program.
 *
 * @author John David Ratliff
 * @version 1.5, 10/22/04
 */
public class CHTApplet extends JApplet {
    private JFrame frame;
    private JFileChooser fileChooser;
    private JPanel buttonPanel;
    private JScrollPane codeScroller;
    private JLabel copyrightLabel;
    private JLabel nameLabel;
    private JButton openButton;
    private JButton saveButton;
    private JPanel textPanel;
    private JList codeList;
    private DefaultListModel codeListModel;
    private static final FileFilter TXT_FILTER = new BasicFileFilter(".txt",
                                                                     "Text Files (*.txt)");
    private static final FileFilter CHT_FILTER = new BasicFileFilter(".cht",
                                                                     "ZSNES CHT Files (*.cht)");

    /**
     * Creates a new CHTApplet object.
     */
    public CHTApplet() {
        this.frame = new JFrame();
    }

    /**
     * Creates a new CHTApplet object.
     *
     * @param frame The parent frame.
     */
    public CHTApplet(JFrame frame) {
        this.frame = frame;
    }

    /**
     * Initializes the applet.
     */
    public void init() {
        initComponents();
    }

    /**
     * Initializes the applet's components.
     */
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        buttonPanel = new JPanel();
        openButton = new JButton();
        saveButton = new JButton();
        textPanel = new JPanel();
        nameLabel = new JLabel();
        copyrightLabel = new JLabel();
        codeScroller = new JScrollPane();
        codeListModel = new DefaultListModel();
        codeList = new JList(codeListModel);
        codeList.setTransferHandler(new TransferHandler() {
                public boolean canImport(JComponent component,
                                         DataFlavor[] flavors) {
                    return (flavors[0].equals(DataFlavor.stringFlavor) ||
                           flavors[0].equals(DataFlavor.javaFileListFlavor));
                }

                public boolean importData(JComponent component,
                                          Transferable data) {
                    try {
                        if (data.isDataFlavorSupported(DataFlavor.stringFlavor)) {
                            String fileURI = ((String)data.getTransferData(DataFlavor.stringFlavor)).trim();

                            if (!fileURI.startsWith("file://")) {
                                // only support local Files...
                                return false;
                            }

                            File chtFile = new File(new URI(fileURI));
                            readCHTFile(chtFile);
                        } else if (data.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
                            List list = (List)data.getTransferData(DataFlavor.javaFileListFlavor);
                            File chtFile = (File)list.get(0);
                            readCHTFile(chtFile);
                        }
                    } catch (URISyntaxException e) {
                        // this seems unlikely...
                        return false;
                    } catch (UnsupportedFlavorException e) {
                        // we check for this, so this exception will not be thrown
                        return false;
                    } catch (IOException e) {
                        // this seems unlikely, but I suppose possible...                
                        return false;
                    }

                    return true;
                }
            });

        getContentPane().setLayout(new GridBagLayout());

        openButton.setText("Open");
        openButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    openButtonAction();
                }
            });
        buttonPanel.add(openButton);

        saveButton.setText("Save");
        saveButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    saveButtonAction();
                }
            });
        buttonPanel.add(saveButton);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.05;
        getContentPane().add(buttonPanel, gridBagConstraints);

        textPanel.setLayout(new GridBagLayout());

        nameLabel.setText(CHTDecoder.APP_NAME + " " +
                          CHTDecoder.APP_VERSION);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        textPanel.add(nameLabel, gridBagConstraints);

        copyrightLabel.setText(CHTDecoder.APP_COPYRIGHT);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        textPanel.add(copyrightLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        getContentPane().add(textPanel, gridBagConstraints);

        codeList.setFont(new Font("Monospaced", Font.PLAIN, 14));
        codeScroller.setViewportView(codeList);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.95;
        getContentPane().add(codeScroller, gridBagConstraints);
    }

    /**
     * Builds the FileDialog used to open and save files.
     */
    private void buildFileDialog() {
        this.fileChooser = new JFileChooser();
    }

    /**
     * Callback when the open button is pressed.
     */
    private void openButtonAction() {
        if (this.fileChooser == null) {
            buildFileDialog();
        }

        this.fileChooser.setFileFilter(CHT_FILTER);

        int result = this.fileChooser.showOpenDialog(this.frame);

        if (result != JFileChooser.APPROVE_OPTION) {
            return;
        }

        File file = this.fileChooser.getSelectedFile();

        if (file.getName().toLowerCase().endsWith(".cht")) {
            this.readCHTFile(file);
        }
    }

    /**
     * Reads a .CHT file and decodes it.
     *
     * @param file The .CHT File.
     */
    protected void readCHTFile(File file) {
        boolean force = false;

        if (!AbstractCHTFileReader.isValidCHTFile(file)) {
            int result = JOptionPane.showConfirmDialog(this.frame,
                                                       "Really Open?",
                                                       "Warning: Potentially Invalid .CHT File",
                                                       JOptionPane.YES_NO_OPTION);

            if (result == JOptionPane.YES_OPTION) {
                force = true;
            }
        }

        try {
            CHTCodeIterator iterator = AbstractCHTFileReader.readCHTFile(file,
                                                                         force);
            this.codeListModel.clear();

            int size = iterator.remain();

            for (int i = 0; i < size; i++) {
                CHTCode code = iterator.next();
                String string = (i + 1) + ": " + code;

                this.codeListModel.addElement(string);
            }
        } catch (CHTFileException e) {
            JOptionPane.showMessageDialog(this.frame, e.getMessage(),
                                          ".CHT File Error",
                                          JOptionPane.ERROR_MESSAGE);
        }
    }

    /**
     * Callback when the save button is pressed.
     */
    private void saveButtonAction() {
        if (this.fileChooser == null) {
            buildFileDialog();
        }

        this.fileChooser.setFileFilter(TXT_FILTER);

        int result = this.fileChooser.showSaveDialog(this.frame);

        if (result != JFileChooser.APPROVE_OPTION) {
            return;
        }

        this.writeTXTFile(this.fileChooser.getSelectedFile());
    }

    /**
     * Writes the CHT codes to a .TXT file.
     *
     * @param file The .TXT File.
     */
    protected void writeTXTFile(File file) {
        try {
            PrintWriter writer = new PrintWriter(new FileWriter(file));
            Enumeration codes = this.codeListModel.elements();

            while (codes.hasMoreElements()) {
                writer.println(codes.nextElement().toString());
            }

            writer.flush();
            writer.close();
        } catch (IOException e) {
            JOptionPane.showMessageDialog(this.frame, e.getMessage(),
                                          "IO Error",
                                          JOptionPane.ERROR_MESSAGE);
        }
    }

    /**
     * Returns a String representation of this applet.
     *
     * @return A String representation.
     */
    public String toString() {
        return "CHTApplet";
    }

    /**
     * Simple file filter class that provides filtering on one extention
     * type.
     *
     * @author John David Ratliff
     * @version 1.1, 09/27/04
     */
    private static class BasicFileFilter extends FileFilter {
        private String extention;
        private String description;

        /**
         * Creates a new BasicFileFilter object.
         *
         * @param extention The extention (lower case, e.g. .txt or .cht)
         * @param description A description of the filter (e.g. TEXT Files
         *        (.txt))
         */
        private BasicFileFilter(String extention, String description) {
            this.extention = extention;
            this.description = description;
        }

        /**
         * Determines whether this Filter will accept this File or not.
         *
         * @param file The File to accept test.
         *
         * @return true if the File passes this Filter; false otherwise.
         */
        public boolean accept(File file) {
            if (file.isDirectory()) {
                return true;
            }

            return file.getName().toLowerCase().endsWith(this.extention);
        }

        /**
         * Gets this filter's description.
         *
         * @return The filter description.
         */
        public String getDescription() {
            return this.description;
        }

        /**
         * Returns a String representation of this FileFilter.
         *
         * @return A String representation.
         */
        public String toString() {
            return "BasicFileFilter[extention=" + this.extention +
                   ", description=" + this.description + "]";
        }
    }
}
